package  
{
	import com.orangesuzuki.util.array.ArrayUtil;
	import com.orangesuzuki.util.timer.TimerUtil;
	import flash.display.Loader;
	import flash.display.MovieClip;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.IOErrorEvent;
	import flash.net.URLRequest;
	import flash.system.ApplicationDomain;
	import flash.system.LoaderContext;
	
	/**
	 * ...
	 * @author Katsushi Suzuki
	 */
	public class GameView extends MovieClip
	{
		private var _deck:Array = [];
		private var _clickCards:Array = [];
		private var _win_num:uint = 0;
		
		private var _motion_good:MovieClip;
		private var _motion_miss:MovieClip;
		private var _motion_clear:MovieClip;
		private var asset:Loader;
		
		public function GameView(iconArray:Array) 
		{
			//素材の読み込み
			var context:LoaderContext = new LoaderContext();  
			context.applicationDomain = ApplicationDomain.currentDomain;
			asset = new Loader();
			asset.contentLoaderInfo.addEventListener(Event.COMPLETE, completeHandler);
			asset.contentLoaderInfo.addEventListener(IOErrorEvent.IO_ERROR, errorHandler);
			asset.load(new URLRequest("asset.swf"), context);
			addChild(asset);
			
			function completeHandler(e:Event):void {
				for (var i:int = 0; i < MovieClip(asset.content).numChildren; i++) 
				{
					var mc:MovieClip = MovieClip(asset.content).getChildAt(i) as MovieClip
					mc.stop();
					mc.visible = false;
				}
				
				//カードを生成します。
				createCards(iconArray);
				
				//正解・不正解の演出を作る
				createMotion();
			}
		}
		
		/**
		 * 正解、不正解のモーション用MCを作成
		 */
		private function createMotion():void
		{
			_motion_good = addChild( Object(asset.content).win_motion) as MovieClip;
			_motion_miss = addChild( Object(asset.content).miss_motion) as MovieClip;
			_motion_clear = addChild( Object(asset.content).clear_motion) as MovieClip;
			_motion_good.gotoAndStop(1);
			_motion_miss.gotoAndStop(1);
			_motion_clear.gotoAndStop(1);
			_motion_good.x = 400;
			_motion_good.y = 260;
			_motion_miss.x = 400;
			_motion_miss.y = 260;
			_motion_clear.x = 400;
			_motion_clear.y = 260;
			_motion_good.visible = false;
			_motion_miss.visible = false;
			_motion_clear.visible = false;
			_motion_good.mouseEnabled = false;
			_motion_good.mouseChildren = false;
			_motion_miss.mouseEnabled = false;
			_motion_clear.mouseEnabled = false;
		}
		
		/**
		 * カードを生成します。
		 */
		private function createCards(iconArray:Array):void
		{
			//カードを配置するベースの作成
			var base:Sprite = addChild(new Sprite()) as Sprite;
			base.x = 60;
			base.y = 60;
				
			var len:uint = Math.min(10, iconArray.length);
			for (var i:int = 0; i < len; i++)
			{					
				//カードの生成
				//1枚目の生成
				var icon:Icon = iconArray[i];
				var card:Card = base.addChild(new Card()) as Card;
				card.init(icon);
				
				//2枚目の生成
				var card2:Card = base.addChild(card.clone()) as Card;
				
				//IDを振る
				card.id = i;
				card2.id = i;
				
				//デッキに格納
				_deck.push( card );
				_deck.push( card2 );
			}
			
			//シャッフル
			ArrayUtil.getShuffleArray(_deck);
			
			//配置
			for (var j:uint=0; j < _deck.length; j++) {
				card = _deck[j];
				card.x = 100 * (j % 7);
				card.y = 155 * Math.floor(j / 7);
				card.addEventListener("onClick", onClickHandler);
			}
		}
		
		
		/**
		 * カードクリック時の処理
		 */
		private function onClickHandler(e:Event):void 
		{
			//めくったカードを配列に入れる
			_clickCards.push(e.target);
			
			//2枚めくったとき
			if ( _clickCards.length == 2) {
				judge();
			}
		}
		
		/**
		 * ゲームの判定処理
		 */
		private function judge():void
		{	
			//正解
			if ( _clickCards[0].id == _clickCards[1].id ) {
				
				//正解ポイントを追加
				_win_num += 2;
				
				//通常正解
				if (_win_num < _deck.length-1) win();
					
				//全問正解
				else allClear();
			}
			//ミス
			else {
				lose();
			}
		}
		
		/**
		 * 正解の処理
		 */
		private function win():void
		{
			_clickCards[0].win();
			_clickCards[1].win();
			_motion_miss.visible = false;
			_motion_good.visible = true;
			_motion_good.gotoAndPlay(2);
			initCards();
		}
		
		/**
		 * 不正解の処理
		 */
		private function lose():void
		{	
			_motion_good.visible = false;
			_motion_miss.visible = true;
			_motion_miss.gotoAndPlay(2);
			unTouchCards();
			TimerUtil.call(initCards, this, null, 1);
		}
		
		/**
		 * 全問正解の処理
		 */
		private function allClear():void
		{
			_clickCards[0].win();
			_clickCards[1].win();
			_motion_good.visible = false;
			_motion_clear.visible = true;
			_motion_clear.gotoAndPlay(2);
		}
		
		
		/**
		 * カードをクリック不可にする
		 */
		private function unTouchCards():void
		{
			for (var i:uint = 0; i < _deck.length; i++) {
				var _card:Card = _deck[i];
				_card.unSetEvent();
			}
		}
		
		/**
		 * カードをクリック可能にする
		 */
		private function initCards():void
		{			
			for (var i:int = 0; i < _clickCards.length; i++) {
				var _card:Card = _clickCards[i];
				if (!_card.end ) _card.flip();
			}
			_clickCards = [];
			
			for (i = 0; i < _deck.length; i++) {
				_card = _deck[i];
				if (!_card.end ) _card.setEvent();
			}
			_motion_miss.visible = false;
		}
		
		/**
		 * エラー時の処理
		 */
		private function errorHandler(e:IOErrorEvent):void 
		{
		}
	}

}