﻿package {
	
	import com.facebook.graph.Facebook;
	import flash.display.MovieClip;
	import flash.events.MouseEvent;
	import flash.system.Security;
	import flash.text.TextField;
	
	/**
	 * Facebookアプリとの接続をテストするクラスです。
	 * 友達一覧を取得します。
	 */
	public class Test4 extends MovieClip
	{
		private var _debug_txt:TextField;
		private var _login_btn:MovieClip;
		private var _friendsDataArray:Array;
		
		public function Test4() 
		{
			Security.loadPolicyFile("https://fbcdn-profile-a.akamaihd.net/crossdomain.xml");
			_debug_txt = this.debug_txt;
			_login_btn = this.login_btn;
			init();
		}
		
		/**
		 * 初期化します。
		 */
		private function init():void 
		{
			debug("Facebook APIの初期化 : ");
			Facebook.init("アプリケーションIDを入れてください", initHandler);
		}
		
		/**
		 * 初期化メソッドのコールバック関数
		 * @param	response
		 * @param	fail
		 */
		private function initHandler(response:Object, fail:Object):void
		{
			if (response) {
				debug("アプリの認証済みです。");
				debug("\nユーザーID : " + response.uid);
				debug("\nアクセストークン ： " + response.accessToken);
				
				//loadIcon(response.uid);
				getFriendList();
			}
			else {
				debug("未ログインまたは未認証です。");
				_login_btn.addEventListener(MouseEvent.CLICK, clickHandler);
				_login_btn.buttonMode = true;
			}
		}
		
		/**
		 * ログインボタンをクリックしたときの処理
		 * @param	e
		 */
		private function clickHandler(e:MouseEvent):void 
		{
			_login_btn.removeEventListener(MouseEvent.CLICK, clickHandler);
			logIn();
		}
		
		/**
		 * ログインします。
		 */
		private function logIn():void 
		{
			debug("ログインします。");
			
            //権限の設定
            var opts = {};
			//opts = { scope:"user_photos" };
            //opts = { scope:"publish_stream,user_photos" };
			
            //ログイン・認証ウィンドウを開く
			Facebook.login(loginHandler, opts);
		}
		
		/**
		 * ログインメソッドのコールバック関数
		 * @param	response
		 * @param	fail
		 */
        private function loginHandler(response:Object, fail:Object = null):void
		{
            if (response) {
                //ログイン・認証済み
                debug("ログイン・認証済み");
				_login_btn.visible = false;
				
				debug("\nユーザーID : " + response.uid);
				debug("\nアクセストークン ： " + response.accessToken);
				
				//loadIcon(response.uid);
				getFriendList();
				
            } else {
                //ログイン・認証失敗
                debug("ログイン・認証失敗");
				_login_btn.visible = true;
				_login_btn.addEventListener(MouseEvent.CLICK, clickHandler);
            }
        }
		
		/**
		 * アイコン画像を読み込んで表示します。
		 * @param	id
		 */
		private function loadIcon(id:String):Icon 
		{
			var url:String = Facebook.getImageUrl(id, "square");
			var icon:Icon = new Icon(url);
			icon.x = 10;
			icon.y = 40;
			addChild(icon);
			return icon;
		}
		
		/**
		 * 友達一覧を取得します。
		 */
		private function getFriendList():void
		{
			debug("\n友達一覧を取得します。");
			Facebook.api("/me/friends", getFriendListHandler);
		}
		
		/**
		 * 友達一覧を取得する際のコールバック関数
		 * @param	response
		 * @param	fail
		 */
		private function getFriendListHandler(response:Object, fail:Object = null):void 
		{
            if (response) {
				_friendsDataArray = [];
				
				debug("取得成功");
                var friends:Array = response as Array;
				var num:uint = Math.min(friends.length, 100);
                for (var i:uint = 0; i < num; i++) {
					_friendsDataArray.push(friends[i]);
                }
				
				for (var j:int = 0; j < 30; j++)
				{
					var friend:Object = _friendsDataArray[j];
					debug(friend.id + ":" + friend.name);
					var icon:Icon = loadIcon(friend.id);
					
					var size:uint = 56;
					icon.x = 10 + j % 5 * size;
					icon.y = 40 + Math.floor( j / 5) * size;
				}
            }else {
				debug("取得失敗");
            }
		}
		
		/**
		 * デバッグ用関数
		 * @param	message
		 */
		private function debug(message:*):void
		{
			_debug_txt.appendText(String(message)+"\n");
		}
	}
}
